/* $Id: generic_addr.c,v 1.4 2002/09/18 13:31:49 bortzmeyer Exp $ */

#include "generic_addr.h"

bool
are_equal (generic_addr left, generic_addr right)
{
  if (!ARE_SAMETYPE (left, right))
    {
      return 0;
    }
  else if (IS_IPV4 (left))
    {
      return (IPV4_ADDR (left).s_addr == IPV4_ADDR (right).s_addr);
    }
  else if (IS_IPV6 (left))
    {
      if (memcmp
	  (&IPV6_ADDR (left).s6_addr, &IPV6_ADDR (right).s6_addr,
	   sizeof (struct in6_addr)) == 0)
	return 1;
      else
	return 0;
    }
  else
    {				/* Should not happen */
      return 0;
    }
}

bool
is_lt (generic_addr left, generic_addr right)
{
  /* IPv4v addresses always come first */
  if (IS_IPV4 (left) && IS_IPV6 (right))
    {
      return 1;
    }
  else if (IS_IPV6 (left) && IS_IPV4 (right))
    {
      return 0;
    }
  else if (IS_IPV4 (left) && IS_IPV4 (right))
    {
      return (ntohl ((IPV4_ADDR (left)).s_addr) <
	      ntohl ((IPV4_ADDR (right)).s_addr));
    }
  else if (IS_IPV6 (left) && IS_IPV6 (right))
    {
      if (memcmp
	  (&IPV6_ADDR (left).s6_addr, &IPV6_ADDR (right).s6_addr,
	   sizeof (struct in6_addr)) < 0)
	return 1;
      else
	return 0;
    }
  else
    {				/* Should not happen */
      return 0;
    }
}

bool
is_le (generic_addr left, generic_addr right)
{
  return (is_lt (left, right) || are_equal (left, right));
}

bool
is_gt (generic_addr left, generic_addr right)
{
  return (!is_le (left, right));
}

bool
is_ge (generic_addr left, generic_addr right)
{
  return (!is_lt (left, right));
}

int
cmp (generic_addr left, generic_addr right)
{
  if (IS_IPV4 (left) && IS_IPV6 (right))
    {
      return -1;
    }
  else if (IS_IPV6 (left) && IS_IPV4 (right))
    {
      return 1;
    }
  else if (IS_IPV4 (left) && IS_IPV4 (right))
    {
      if (ntohl (IPV4_ADDR (left).s_addr) < 
	  ntohl (IPV4_ADDR (right).s_addr))
	{
	  return -1;
	}
      else if (IPV4_ADDR (left).s_addr == IPV4_ADDR (right).s_addr)
	{
	  return 0;
	}
      else
	{
	  return 1;
	}
    }
  else if (IS_IPV6 (left) && IS_IPV6 (right))
    {
      return (memcmp
	      (&IPV6_ADDR (left).s6_addr, &IPV6_ADDR (right).s6_addr,
	       sizeof (struct in6_addr)));
    }
  else
    {				/* Should not happen */
      return 0;
    }
}

generic_addr
text_to_address (char *textaddress)
{
  int result;
  generic_addr gaddr;
  INIT (gaddr);
  result = inet_pton (AF_INET6, textaddress, &IPV6_ADDR (gaddr));
  if (result)
    {
      gaddr.type = AF_INET6;
    }
  else
    {
      result = inet_pton (AF_INET, textaddress, &IPV4_ADDR (gaddr));
      if (result)
	{
	  gaddr.type = AF_INET;
	}
      else
	{
	  gaddr.type = AF_UNSPEC;
	}
    }
  return gaddr;
}

char *
address_to_text (generic_addr gaddr)
{
  const char *cresult;
  char *textaddress;
  if (IS_IPV6 (gaddr))
    {
      textaddress = (char *) MALLOC (INET6_ADDRSTRLEN);
      cresult = inet_ntop (AF_INET6, &IPV6_ADDR (gaddr),
			   textaddress, INET6_ADDRSTRLEN);
      if (!cresult)
	{
	  strcpy (textaddress, "INVALID ADDRESS");
	}
    }
  else if (IS_IPV4 (gaddr))
    {
      textaddress = (char *) MALLOC (INET_ADDRSTRLEN);
      cresult = inet_ntop (AF_INET, &IPV4_ADDR (gaddr),
			   textaddress, INET_ADDRSTRLEN);
      if (!cresult)
	{
	  strcpy (textaddress, "INVALID ADDRESS");
	}
    }
  else
    {
      strcpy (textaddress, "UNKNOWN ADDRESS TYPE");
    }
  return textaddress;
}
