/* Definition of a generic IP address type, which can be IPv4 or IPv6 */
/* $Id: generic_addr.h,v 1.2 2002/09/18 09:08:54 bortzmeyer Exp $ */

#ifndef GENERIC_ADDR_DEFINED

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>

/* It is highly recommanded to use the macros and not the fields of
   generic_addr */
typedef struct
{
  int type;			/* An address family, AF_INET or AF_INET6 */
  union
  {
    struct in_addr ipv4_addr;
    struct in6_addr ipv6_addr;
  }
  actual_addr;
}
generic_addr;

/* TODO: these macros should test the type! Not easy in C */
#define IPV6_ADDR(a) (a).actual_addr.ipv6_addr
#define IPV4_ADDR(a) (a).actual_addr.ipv4_addr
#define IS_IPV4(a) ((a).type == AF_INET)
#define IS_IPV6(a) ((a).type == AF_INET6)
#define IS_UNSPEC(a) ((a).type == AF_UNSPEC)
#define ARE_SAMETYPE(a, b) ((a).type == (b).type)
#define INIT(a) ((a).type = AF_UNSPEC,memset(&(a),0,sizeof(generic_addr)))

/* Comparison functions */
#ifndef bool
typedef char bool;
#endif
bool are_equal (generic_addr left, generic_addr right);
bool is_lt (generic_addr left, generic_addr right);
bool is_le (generic_addr left, generic_addr right);
bool is_gt (generic_addr left, generic_addr right);
bool is_ge (generic_addr left, generic_addr right);
int cmp (generic_addr left, generic_addr right);

/* Input-Output functions */
generic_addr text_to_address (char *textaddress);
char *address_to_text (generic_addr gaddr);	/* Do not forget to free() it */

#define GENERIC_ADDR_DEFINED 1
#endif
